import configparser
import os
import vcr
from pyasan import helpers
from pytest import fixture

def test_create_config_successfull(tmpdir):
    """Tests that creation of config.ini is successfull"""

    fh = tmpdir.join("config.ini")
    fh = str(fh)
    helpers.create_config(fh)


    config = configparser.ConfigParser()

    config.read(fh)

    assert config.sections() == ['Global', 'PATENT']

def test_get_config_successfull(tmpdir):
    fh = tmpdir.join('config.ini')
    fh = str(fh)
    helpers.get_config(fh)

    config = configparser.ConfigParser()

    config.read(fh)

    assert os.path.exists(fh)
    assert config.sections() == ['Global', 'PATENT']

def test_init_config_successfull():
    config = helpers.init_config()

    assert config == configparser.ConfigParser()

def test_get_app_token_successfull():
    app_token = helpers.get_app_token()

    assert app_token == ''

def test_get_url_successfull():
    patent_url = helpers.get_url('PATENT')

    assert patent_url == 'https://data.nasa.gov/resource/nasa-patents.json'


@fixture
def patent_keys():
    #Responsible only for returning the test data.
    return ['patent_number', 'center', 'patent_expiration_date',
            'case_number', 'title', 'application_sn',
            'status']

@vcr.use_cassette('tests/vcr_cassettes/api_get.yml')
def test_api_get_successful(patent_keys):
    """Tests and API call to return a patent from the NASA API"""

    test_url = 'https://data.nasa.gov/resource/nasa-patents.json'
    test_payload = {'patent_number': '5694939'}
    test_response = helpers.api_get(test_url, test_payload)

    assert isinstance(test_response[0], dict)
    assert test_response[0]['patent_number'] == '5694939', 'The patent_number should be in the response'
    assert set(patent_keys).issubset(test_response[0].keys()), 'All keys should be in the response'


from pyasan import patents, Patents
import vcr
from pytest import fixture

@fixture
def patent_keys():
    #Responsible only for returning the test data.
    return ['patent_number', 'center', 'patent_expiration_date',
            'case_number', 'title', 'application_sn',
            'status']

@vcr.use_cassette('tests/vcr_cassettes/Patent_req.yml')
def test_Patent_req_successful(patent_keys):
    """Tests and API call to return a patent from the NASA API"""
    
    patent_params = {'$q': 'fuel'}
    p = Patents(params=patent_params)
    test_query = p.req()

    assert isinstance(test_query[1], dict)
    assert set(patent_keys).issubset(test_query[1].keys()), 'All keys should be in the response'
    assert 'fuel' in test_query[1]['title'].lower(), 'Query should be in the title'

@vcr.use_cassette('tests/vcr_cassettes/Patent_req.yml')
def test_patent_get_successful(patent_keys):
    """Tests get call to query patent database from NASA API"""

    test_get = patents.get(query='fuel')

    assert isinstance(test_get[1], dict)
    assert set(patent_keys).issubset(test_get[1].keys()), 'All keys should be in the response'
    assert 'fuel' in test_get[1]['title'].lower(), 'Query should be in the title'





from . import helpers
import pprint


class Patents(object):
    def __init__(self, url=helpers.get_url('PATENT'), params=None, **kwargs):
        self.url = url
        self.params = params

    def req(self):

        self.response = helpers.api_get(self.url, self.params)
        return self.response

    def __str__(self):
        return pprint.pformat(self.req())


def get(query=None, patent_num=None,
        center=None, patent_exp=None,
        case_num=None, title=None,
        app_sn=None, status=None, **kwargs):

    params = {'$q': query,
              'patent_number': patent_num,
              'center': center,
              'case_number': case_num,
              'title': title,
              'application_sn': app_sn,
              'status': status}

    pat = Patents(params=params)

    return pat.req()



# -*- coding: utf-8 -*-

"""Top-level package for pyasan."""

__author__ = """Jeorry Balasabas"""
__email__ = 'jeorry@gmail.com'
__version__ = '0.1.0'

from . import helpers
from . import patents
from .patents import Patents



import configparser
import os
import requests


class NasaApiException(Exception):
    """Raised for any exception caused by a call to the Nasa API"""


def create_config(path):
    """
    Create pyasan config file
    """
    config = configparser.ConfigParser()
    config.add_section('Global')
    config.set('Global', 'APP_TOKEN', '')
    config.add_section('PATENT')
    config.set('PATENT', 'URL',
               'https://data.nasa.gov/resource/nasa-patents.json')

    with open(path, 'w') as config_file:
        config.write(config_file)


def get_config(path):
    """
    Return config object.
    """
    if not os.path.exists(path):
        create_config(path)
    config = configparser.ConfigParser()
    return config


def init_config():
    path = 'config.ini'
    config = get_config(path)
    return config


def get_app_token():
    config = init_config()
    config.read('config.ini')
    app_token = config['Global']['app_token']
    return app_token


def get_url(api):
    """Grabs URL from config.ini for the corresponding API."""
    config = init_config()
    config.read('config.ini')
    url = config[api]['url']
    return url


def app_token_exists():
    token = get_app_token()
    if token != '':
        return True


def api_get(url, payload):
    payload = dict((k, v) for k, v in payload.items() if v)
    if app_token_exists():
        payload['$$app_token'] = get_app_token()
    response = requests.get(url, params=payload)
    response.raise_for_status()
    body = response.json()

    if 'error' in body:
        raise NasaApiException(body['error'])

    return body
